--
-- manuallyPowerShaft
-- attach only pto without attaching first. This script use parts from PowerShaft specialization written by Manuel Leithner.
--
-- author:    	Xentro (www.ls-uk.info)
-- @version:    v1.0
-- @date:       2012-08-26 
-- @history:    v1.0 - inital implementation
-- 
--
manuallyPowerShaft = {};

function manuallyPowerShaft.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(PowerShaft, specializations);
end;

function manuallyPowerShaft:load(xmlFile)
	self.allowNewDraw = self;
	-- self.FMCDraw = manuallyPowerShaft.FMCDraw;
	self.getVehicle = manuallyPowerShaft.getVehicle;
	self.addDraw = manuallyPowerShaft.addDraw;
	self.deleteDraw = manuallyPowerShaft.deleteDraw;
	
	self.inRange = manuallyPowerShaft.inRange;
	self.detachPto = SpecializationUtil.callSpecializationsFunction("detachPto");
	
	self.ptoShaft = {};
	
	self.distance = {};
	self.distance.player = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.manuallyPowerShaft#player"), 4);		-- pto 		=> player
	self.distance.vehicle = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.manuallyPowerShaft#vehicle"), 10);		-- vehicle	=> pto
	self.distance.joint = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.manuallyPowerShaft#joint"), 2.55);		-- joint	=> pto
	local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile,  "vehicle.manuallyPowerShaft#rotLimit"));
	self.distance.rotX = Utils.getNoNil(x, 0);		-- joint	=> pto   how far you can X rotate when attached
	self.distance.rotY = Utils.getNoNil(y, 0);		-- joint	=> pto   how far you can Y rotate when attached
	self.distance.rotZ = Utils.getNoNil(z, 0);		-- joint	=> pto   how far you can Z rotate when attached
end;

function manuallyPowerShaft:delete()
end;

function manuallyPowerShaft:mouseEvent(posX, posY, isDown, isUp, button)
end;

function manuallyPowerShaft:keyEvent(unicode, sym, modifier, isDown)
end;

function manuallyPowerShaft:update(dt)	
	if self.attacherVehicle == nil and g_currentMission.player ~= nil then
		if self:inRange(self.powerShaft.fixPoint, g_currentMission.player.rootNode, self.distance.player) then
			if self.ptoShaft.vehicle == nil then
				for _, vehicle in pairs(g_currentMission.vehicles) do
					if vehicle ~= self then
						if self:inRange(vehicle.components[1].node, self.powerShaft.fixPoint, self.distance.vehicle) then
							for _, joint in pairs(vehicle.attacherJoints) do
								if joint.jointType == self.attacherJoint.jointType then
									if self:inRange(joint.jointTransform, self.PTOdeattached, self.distance.joint) and not self:getIsActive() then
										if InputBinding.hasEvent(InputBinding.ATTACH) then
											self.ptoShaft.vehicle = vehicle;
											self.ptoShaft.vehicleJoint = joint.jointTransform;
											if joint.powerShaftAttacher ~= nil then
												self.ptoShaft.joint = joint.powerShaftAttacher;
											end;
											
											self:setPTO(false);
										end;
										
										g_currentMission:addHelpButtonText(string.format(g_i18n:getText("ATTACHPTO"), self.typeDesc), InputBinding.ATTACH);
									end;
								end;
							end;
						end;
					end;
				end;
			else
				if not self:getIsActive() then
					if InputBinding.hasEvent(InputBinding.ATTACH) then
						self:setPTO(true);
						self:detachPto();
					end;
					g_currentMission:addHelpButtonText(string.format(g_i18n:getText("DETACHPTO"), self.typeDesc), InputBinding.ATTACH);
				end;
			end;
		end;
		
		if self.ptoShaft.vehicleJoint ~= nil then
			local x, y, z = getRotation(self.powerShaft.node);
			
			local detach = false;
			if self.distance.rotX ~= 0 and x > self.distance.rotX and x < -self.distance.rotX then detach = true; end;
			if self.distance.rotY ~= 0 and y > self.distance.rotY and y < -self.distance.rotY then detach = true; end;
			if self.distance.rotZ ~= 0 and z > self.distance.rotZ and z < -self.distance.rotZ then detach = true; end;
			if not self:inRange(self.ptoShaft.vehicleJoint, self.PTOdeattached, self.distance.joint) then detach = true; end;
			
			if detach then self:detachPto(); end;
		end;
		
		if self.ptoShaft.joint ~= nil then
			local ax, ay, az = getWorldTranslation(self.powerShaft.node);
			local bx, by, bz = getWorldTranslation(self.ptoShaft.joint);
			local x, y, z = worldDirectionToLocal(getParent(self.powerShaft.node), bx-ax, by-ay, bz-az);
			setDirection(self.powerShaft.node, x, y, z, 0, -1, 0);
			local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
			setTranslation(self.powerShaft.part, 0, 0, distance-self.powerShaft.distance);		
		end;
	end;
end;

function manuallyPowerShaft:updateTick(dt)	

end;

function manuallyPowerShaft:draw()
end;

-- function manuallyPowerShaft.FMCDraw(self, attachedToVehicle)
-- end;

function manuallyPowerShaft:onAttach()
	if self.ptoShaft.vehicle ~= nil then
		if self.ptoShaft.joint ~= nil then
			self.attacherVehiclePowerShaft = self.ptoShaft.joint
		end;
		
		self.ptoShaft.vehicle = nil;
		self.ptoShaft.vehicleJoint = nil;
		self.ptoShaft.joint = nil;
		
		self:setPTO(false);
	end;
end;

function manuallyPowerShaft:detachPto()
	self.ptoShaft.vehicle = nil;
	self.ptoShaft.vehicleJoint = nil;
	self.ptoShaft.joint = nil;
	
	setTranslation(self.powerShaft.part, unpack(self.powerShaft.trans));
	setRotation(self.powerShaft.node, unpack(self.powerShaft.rot));
	
	if not self.PTOId then
		self:setPTO(true);
	end;
end;

function manuallyPowerShaft:inRange(object, object2, nd)
	local x1,y1,z1 = getWorldTranslation(object);
	local x2,y2,z2 = getWorldTranslation(object2);
	local distance = Utils.vector3Length(x1-x2,y1-y2,z1-z2);
	if distance < nd then
		return true;
	else
		return false;
	end;
end;

function manuallyPowerShaft:getVehicle()
	if self.ptoShaft.vehicle ~= nil then
		return self.ptoShaft.vehicle;
	end;
	if self.attacherVehicle ~= nil then
		return self.attacherVehicle;
	end;
	
	return nil
end;

function manuallyPowerShaft:addDraw()
	if self.ptoShaft.vehicle ~= nil or self.attacherVehicle ~= nil then
		return self;
	end;
	
	return nil;
end;

function manuallyPowerShaft:deleteDraw()
	if self.ptoShaft.vehicle ~= nil or self.attacherVehicle ~= nil then
		return nil;
	end;
	
	return self;
end;